% N snails move at velocity v cm per minute, starting along the vertices of
% a regular N-gon. They always move in order to approach the snail ahead of
% them. This means the N-gon geometry always defines their relative
% separations, but the N-gon rotates at rate v*sin(alpha)/r and its radius
% decreases at rate v*cos(alpha). alpha = 90*(N-2)/N i.e. the angle between
% the side of the N-gon and the radial line.
%
% Inspired by Question 1 of "200 Puzzling Physics Problems" by Gnadig,
% Honyek & Riley.
%
% Run code as snails(N). N=5 if no argument supplied.
function snails(varargin)
if nargin==0; N = 5; else N = varargin{1}; end

%Fixed inputs
v = 5;        %Snail speed in cm per minute
s = 60;       %Side length of N-gon /cm
P = 1000;     %Define number of time steps for trajectory plot
dtheta =...
    2*pi/100; %Define angular step size for whirl plot
max_num_whirls =...
    5;        %Maximum number of whirl rotations
fsize = 18;   %Fontsize for graphs
lwidth = 1.5; %Linewidth for plots
show_ax = 0;  %Show axes?

%%

%angle alpha (in radians)
alpha = (pi/2)*(N-2)/N;

%Compute initial radius from cenre of N-gon
r0 = (s/2)/cos(alpha);

%Compute initial angles of the snails (anticlockwise from hirozontal);
for n=1:N
    theta0(n) = pi/2 + (n-1)*2*pi/N;
end

%Compute time required for snails to reach the centre /mins
T = r0/(v*cos(alpha));

%Define trajectory for snails
t = linspace(0,T,P); x = zeros(P,N); y = zeros(P,N);
for n=1:N
    r = r0 - v*t*cos(alpha);
    theta = theta0(n) - tan(alpha) * log( r/r0 );
    x(:,n) = r(:).*cos(theta(:));
    y(:,n) = r(:).*sin(theta(:));
end

%

%Plot initial N-gon
figure('name','snails of pursuit','color',[1 1 1]);
plot( [x(1,:),x(1,1)],[y(1,:),y(1,1)],'k','linewidth',lwidth); hold on;
axis equal; set(gca, 'fontsize',fsize);
xlabel('x /cm','fontsize',fsize); ylabel('y /cm','fontsize',fsize);
title( {['Snails of pursuit around a ',num2str(N),...
    '-gon.'],['T=',num2str(T,3),'mins, v=',num2str(v),...
    'cm/min, s=',num2str(s),'cm.' ] },'fontsize',fsize );
box on; grid on; axis tight;
if show_ax~=1; axis off ; end

%Rotate through angular step size dtheta and work out new snail positions.
%Join these up with straight lines to form a rotated and reduced size
%N-gon.
delta_theta = 0; xx = zeros(1,N+1); yy = zeros(1,N+1);
while delta_theta < max_num_whirls*2*pi
    delta_theta = delta_theta + dtheta;
    for n=1:N
        theta = theta0(n) + delta_theta;
        r = r0*exp( - delta_theta*cot(alpha) );
        xx(n) = r.*cos(theta); yy(n) = r.*sin(theta);
    end
    xx(N+1) = xx(1); yy(N+1) = yy(1);
    plot( xx,yy ,'k','linewidth',lwidth );
end

%Overlay trajectories
plot(x,y,'linewidth',lwidth);

%Plot starting positions
warning off
plot( [x(1,:);x(1,:)],[y(1,:);y(1,:)],'*');

%Save PNG file and close figure
print( gcf, ['Snails of pursuit around a ',num2str(N),...
    '-gon.png' ],'-dpng','-r300' );
close(gcf);

%End of code